import cluttergtk   # must be the first to be imported
import clutter
import gtk

# tempory
import time

import globals
from game import Game
import game

class Selection:
    "current thing selected"

    def __init__ (self, gx, gy, game):
        self.gx = gx
        self.gy = gy

        self.unit = game.get_unit(gx, gy) # the unit is at the pos
        self.can_go = game.can_go_from(gx, gy)
        self.can_attack = game.can_attack_from(gx, gy)

class BoardWidget:
    def __init__(self, theme, game = None, side = None):
        self.unit_actor = {}
        # FIXME : HORROR
        self.BAD = []
        self.selected = None

        self.theme = theme

        # If we are the client, we get passed the current game,
        # otherwhise we create it
        if not game:
            self.game = Game()
        else:
            self.game = game

        # if a side is given we will assume the player behind the UI can only
        # play this side
        self.side = side

        # Clutter init
        self.screen_size = self.theme.screen_size
        self.widget = cluttergtk.Embed()
        self.widget.set_size_request(*self.screen_size)

        self.label = clutter.Label()

    def show(self):
        # we need to realize the widget before we get the stage
        self.widget.realize()

        self.stage = self.widget.get_stage()
        self.stage.set_color(clutter.Color(*self.theme.background_color))
        self.stage.set_size(*self.screen_size)
        self.stage.show()


        # create label for who is playing
        self.turn_text = clutter.Label()
        self.turn_text.set_font_name('Sans 25')
        # TODO : Add a color in the theme for the text
        # FIXME lol wat strange magic number ?
        text_xpos = self.screen_size[0]/2 - self.turn_text.get_size()[0] - 100
        text_ypos = self.screen_size[1] - self.turn_text.get_size()[1]
        self.turn_text.set_color(clutter.Color(*self.theme.mark_color))
        self.turn_text.set_position(text_xpos, text_ypos)
        self.stage.add(self.turn_text)

        self.draw_background()
        self.game.connect("initial_placement", self.initial_placement_cb)
        self.game.connect("unit_moved", self.unit_moved_cb)
        self.game.connect("saboteur_return", self.saboteur_return_cb)
        self.game.connect("ranged_attack", self.ranged_attack_cb)
        self.game.connect("direct_attack", self.direct_attack_cb)
        self.game.connect("player_wins", self.player_wins_cb)
        self.game.connect("turn_ended", self.update_turn_text)
        self.game.connect("new_game", self.new_game_cb)
        self.game.connect("new_game", self.hide_label)

        # Need to connect the signal before launching a new game
        # otherwhise starting units won't be shown
        self.game.new_game()

        self.stage.connect('button-press-event', self.on_button_press_event)

    def on_button_press_event(self, actor, event):
        print self.screen_to_game(event.x, event.y)
        self.clic((event.x, event.y), event.button)

    def game_to_screen(self, *args):
        "resize and re-position in game positions to screen position"
        return [(x+1) * self.theme.case for x in args]

    def screen_to_game(self, *args):
        "Convert screen position to board case"
        return [(x / self.theme.case) -1 for x in args]

    def draw_background (self):
        #TODO clean this shit. move to a class?
        import cairo
        from cluttercairo import CairoTexture
        background_surface = CairoTexture(*self.screen_size)
        self.stage.add(background_surface)
        self.bs = background_surface
        self.bs.lower_bottom()

        case_size = self.theme.case
        border_size = 1 #Should be themable

        cr = background_surface.cairo_create()
        cr.set_operator(cairo.OPERATOR_CLEAR)
        cr.paint()
        cr.set_operator(cairo.OPERATOR_OVER)
        cr.set_line_width(0.5)

        cr.set_source_color(clutter.Color(*self.theme.mark_color))

        def draw_line(x1, y1, x2, y2):
            cr.move_to(x1, y1)
            cr.line_to(x2, y2)
            cr.stroke()

        for i in range(-1, globals.TOTAL_WIDTH):
            x1, y1 = self.game_to_screen(i, -1)
            x2, y2 = self.game_to_screen(i, globals.TOTAL_HEIGHT)
            draw_line(x1, y1, x2, y2)
        for i in range(-1, globals.TOTAL_HEIGHT):
            x1, y1 = self.game_to_screen(-1, i)
            x2, y2 = self.game_to_screen(globals.TOTAL_WIDTH, i)
            draw_line(x1, y1, x2, y2)

        cr.set_line_width(2)

        def draw_rect(x1, y1, x2, y2):
            print x1, y1, x2, y2
            cr.move_to(x1, y1)
            cr.line_to(x1, y2)
            cr.line_to(x2, y2)
            cr.line_to(x2, y1)
            cr.line_to(x1, y1)
            cr.stroke()

        # draw the boards
        draw_rect(*self.game_to_screen(*globals.WHITE_DIM))
        draw_rect(*self.game_to_screen(*globals.BLACK_DIM))
        draw_rect(*self.game_to_screen(*globals.BATTLEFIELD_DIM))

        # draw the hills
        for x, y in globals.HILLS:
            sx, sy = self.game_to_screen(x, y)
            cr.rectangle(sx, sy, self.theme.case, self.theme.case)
            cr.fill()
            cr.stroke()

# function callback and helpers
    def update_turn_text(self, game, turn, turn_number):
        "Update the 'side that play currently' text"
        self.turn_text.set_text(turn+": " + str(turn_number))

    def player_wins_cb(self, object_slot, player):
        "End of the game: display the winner"
        print _("%s wins.") % player  # should be on debug ?
        self.display_winner(player)

    def display_winner(self, player):
        "Display a Label for the winner side"
        self.label.set_color(clutter.Color(255, 255, 255))
        self.label.set_font_name("Sans 25")
        self.label.set_text("%s win the game" % player)
        self.label.set_position((self.screen_size[0] - self.label.get_size()[0])/2,
                (self.screen_size[1] - self.label.get_size()[1])/2)
        self.stage.add(self.label)
        self.label.show()

    def hide_label(self, sender):
        self.label.hide()

    def unit_moved_cb(self, game, unit, gx, gy, from_gx, from_gy):
        print "Unit %s moved from (%i, %i) to (%i, %i)"\
                % (unit, from_gx, from_gy, gx, gy)
        self.unit_actor[unit].set_position(*self.game_to_screen(gx, gy))

    def initial_placement_cb(self, game, unit, gx, gy):
        """This signal is fired when an unit is placed for the first time on
        the board, so we will use the oportunity to create the corresponding
        actor and register it in a dictionnary"""

        name, team = unit.NAME, unit.side
        print "Unit %s placed at (%i, %i)"\
                % (unit, gx, gy)
        image = clutter.Texture()
        image.set_from_file(self.theme.image_source(name, team))
        image.set_position(*self.game_to_screen(gx, gy))
        self.stage.add(image)
        self.unit_actor[unit] = image

    def saboteur_return_cb(self, game, unit, to_x, to_y, from_x, from_y):
        "Make the saboteur unit reappear at his old position"
        print "%s returned to home base at (%i,%i) from  (%i, %i)"\
                % (unit, to_x, to_y, from_x, from_y)
        self.unit_actor[unit].set_position(*self.game_to_screen(to_x, to_y))
        self.unit_actor[unit].show()

    def ranged_attack_cb(self, game, unit, to_x, to_y, victim, from_x, from_y):
        print "Ranged attack from (%i,%i) by %s  on  %s(%s, %s)"\
                % (from_x, from_y, unit, victim, to_x, to_y)
        # Remove attacked unit
        self.unit_actor[victim].hide()

    def direct_attack_cb(self, game, unit, to_x, to_y, victim, from_x, from_y):
        print "Direct attack from (%i,%i) by %s  on  %s(%s, %s)"\
                % (from_x, from_y, unit, victim, to_x, to_y)
        # Some nice animation and sound would be cool
        #Remove attacked unit
        self.unit_actor[victim].hide()
        # Move attacking unit
        self.unit_actor[unit].set_position(*self.game_to_screen(to_x, to_y))

    def new_game_cb(self, game):
        if self.selected:
            self.cancel_selection()

        print 'Clearing the stage'
        for unit, actor in self.unit_actor.items():
            print 'Removing ', unit, actor
            self.stage.remove(actor)
        self.unit_actor = {}

# Commands (move, attacks, end turn, select unit, cancel selection, quit)
    def move_selected_to(self, gx, gy):
        "Move selected unit to position"
        selected = self.finish_selection()
        self.game.move(gx, gy, selected.gx, selected.gy)

    def attack_unit(self, gx, gy):
        "Selected unit attack unit at this position"
        selected = self.finish_selection()
        self.game.attack(gx, gy, selected.gx, selected.gy)

    def select_unit(self, gx, gy):
        "Select the unit on this board and position"
        if self.game.can_act(gx, gy):
            self.selected = Selection(gx, gy, self.game)

            # show that the unit is selected
            actor = self.unit_actor[self.game.get_unit(gx, gy)]
            timeline = clutter.Timeline(fps=60, duration=3000)
            timeline.set_loop(True)
            alpha = clutter.Alpha(timeline, clutter.sine_func)

            g_behaviour = clutter.BehaviourScale(alpha=alpha,
                    x_scale_start= 1.1, x_scale_end = 0.9, y_scale_start = 1.1,
                    y_scale_end = 0.9)
            g_behaviour.apply(actor)

            timeline.start()
            self.timeline = timeline
            self._behaviour = g_behaviour

            # Mark where the unit can move or attack
            def mark_case( x, y, color):
                "Draw an overlay on the case in whatever color given"
                sx, sy = self.game_to_screen(x, y)
                rect = clutter.Rectangle()
                rect.set_position(sx+2, sy+3)
                rect.set_size(self.theme.case -5, self.theme.case -5)
                rect.set_color(clutter.Color(*color, alpha = 50))

                self.stage.add(rect)
                self.BAD.append(rect)

            for x, y in self.selected.can_go:
                mark_case(x, y, (0, 255, 255))
            for x, y in self.selected.can_attack:
                mark_case(x, y, (255, 0, 0))

    def cancel_selection(self):
        "cancel selection and put back the unit back in its place"
        if not self.selected:
            raise globals.NothingSelected
        self.finish_selection()

    def finish_selection(self):
        "End a selection : remove the variable and redraw the cursor"
        #stop the animation and give back it size to the unit
        self.timeline.stop()
        self.unit_actor[self.selected.unit].set_scale(1,1)

        # Cancel selection
        selected = self.selected
        self.selected = None

        # Remove all the possible movement case
        for rect in self.BAD:
            self.stage.remove(rect)
            del rect
        self.BAD = []

        return selected

    def end_turn (self, sender = None):
        "give hand to the other player"
        if self.side and self.side != self.game.turn:
            # If it's not our turn do nothing
            return
        if not self.selected:
            self.game.end_turn()
        else:
            print _("Cannot end turn when an unit is selected")
            print "\a" # system beep

# Mouse gestion helpers
    def case_under(self, position):
        "return the case (gx and gy) under the mouse position"
        gx, gy = self.screen_to_game(position[0], position[1])
        return gx, gy

# Events handler
    def clic(self, position, button):
        "Manage a mouse click"
        if self.side and self.side != self.game.turn:
            # If it's not our turn do nothing
            return

        if button == globals.LEFT_MOUSE_BUTTON:
            gx, gy =  self.case_under(position)
            if globals.position_is_valid(gx, gy):
                # If we are on a valid case
                if not self.selected:
                    # if no unit is selected, grab one
                    self.select_unit(gx, gy)
                elif self.selected:
                    # else try to place it on the board
                    if (gx, gy) in self.selected.can_go:
                        # if it's a valid movement
                        self.move_selected_to(gx, gy)
                    elif (gx, gy) in self.selected.can_attack:
                        # if it's a valid attack
                        self.attack_unit(gx, gy)

        elif button == globals.RIGHT_MOUSE_BUTTON:
            if self.selected != None:
                # if a unit is selected, unselect it
                self.cancel_selection()
            else :
                # FIXME : For debug remove that later when we have a button
                self.end_turn()

    # Unused we should some who port this to clutter
    def key (self, presskey, mod):
        "manage the keyboard events"
        actions = {K_RETURN : self.end_turn, K_KP_ENTER: self.end_turn,
                K_f : self.switch_show_FPS}

        if presskey == K_ESCAPE: # always able to quit
            self.quit()
        elif self.display_message :
            # Erase message if one is displayed
            self.display_message = None
        elif presskey in actions.keys():
            # Else if the key is asociated to a action
            actions[presskey]()

