import connection_watcher
import pygtk
import gtk
import logging
import gobject
logging.basicConfig(level=logging.DEBUG)

from tp import conn_can_request_contact_list
from telepathy.interfaces import CHANNEL_TYPE_CONTACT_LIST,\
    CONNECTION_INTERFACE_REQUESTS, CONNECTION_INTERFACE_CONTACTS,\
    CHANNEL_INTERFACE_GROUP, CHANNEL_INTERFACE, CONNECTION_INTERFACE_ALIASING,\
    CONNECTION_INTERFACE_SIMPLE_PRESENCE
from telepathy.constants import HANDLE_TYPE_LIST, HANDLE_TYPE_CONTACT,\
    CONNECTION_PRESENCE_TYPE_AVAILABLE, CONNECTION_PRESENCE_TYPE_AWAY,\
    CONNECTION_PRESENCE_TYPE_EXTENDED_AWAY, CONNECTION_PRESENCE_TYPE_BUSY,\
    TUBE_STATE_OPEN
from telepathy.client.channel import Channel
from telepathy.client.conn import Connection

ALIAS_ATTRIBUTE = 'org.freedesktop.Telepathy.Connection.Interface.Aliasing/alias'
SIMPLE_PRESENCE_ATTRIBUTE = 'org.freedesktop.Telepathy.Connection.Interface.SimplePresence/presence'
ID_ATTRIBUTE = 'org.freedesktop.Telepathy.Connection/contact-id'

from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)

class HConnection(gobject.GObject):
    __gsignals__ = {
        "contacts_added" : (gobject.SIGNAL_RUN_FIRST, None, [object]),
        "contacts_removed" : (gobject.SIGNAL_RUN_FIRST, None, [object]),
        }

    def __init__(self, conn):
        gobject.GObject.__init__(self) # Call the parent constructeur
        self.conn = conn
        self.contacts = {}
        # get contacts list
        yours, path, props = conn[CONNECTION_INTERFACE_REQUESTS].EnsureChannel({
            'org.freedesktop.Telepathy.Channel.ChannelType': CHANNEL_TYPE_CONTACT_LIST,
            'org.freedesktop.Telepathy.Channel.TargetHandleType': HANDLE_TYPE_LIST,
            'org.freedesktop.Telepathy.Channel.TargetID': 'subscribe'})

        self.chan = Channel(conn.service_name, path)
        self.chan[CHANNEL_INTERFACE_GROUP].connect_to_signal('MembersChanged',
            self.members_changed_cb)

    def get_contacts(self):
        "get the contact the first time"
        contacts = self.chan[CHANNEL_INTERFACE_GROUP].GetMembers()
        self.add_contacts(contacts)


    def add_contacts(self, contacts):
        #print conn['org.freedesktop.DBus.Properties'].Get(CONNECTION_INTERFACE_CONTACTS,
                #'ContactAttributeInterfaces')

        ret = []
        # Query infos about the contact.
        r = self.conn[CONNECTION_INTERFACE_CONTACTS].GetContactAttributes(contacts,
            [CONNECTION_INTERFACE_ALIASING,
            CONNECTION_INTERFACE_SIMPLE_PRESENCE], True)
        for contact, infos in r.items():
            handle = int(contact)
            alias = infos[ALIAS_ATTRIBUTE]
            id = infos[ID_ATTRIBUTE]
            presence = infos[SIMPLE_PRESENCE_ATTRIBUTE][1]
            presence_message = infos[SIMPLE_PRESENCE_ATTRIBUTE][2]
            print handle, alias, id, presence, presence_message
            ret.append( (handle, id, alias, presence, presence_message) )
            self.contacts[handle] = (id, alias, presence, presence_message)

        self.emit('contacts_added', ret)


    def members_changed_cb(self, message, added, removed, local_pending,
            remote_pending, actor, reason):
        # Remove removed contact
        removed_handles = []
        for contact_handle in removed:
            contact_handle = int(contact_handle)
            del self.contacts[contact_handle]
            removed_handels.append(contact_handle)
        self.emit('contact_removed', removed_handles)

        # Add new contacts
        self.add_contacts(added)

#register as a gobject
gobject.type_register(HConnection)

def contact_is_online(handle, conn):
    presences = conn[CONNECTION_INTERFACE_SIMPLE_PRESENCE].GetPresences([handle])

    type, status, msg = presences[handle]
    return type in [CONNECTION_PRESENCE_TYPE_AVAILABLE,
            CONNECTION_PRESENCE_TYPE_AWAY,
            CONNECTION_PRESENCE_TYPE_EXTENDED_AWAY, CONNECTION_PRESENCE_TYPE_BUSY]

class ContactManager(gtk.ListStore):
    def __init__(self):
        cols = (int, str, str, str, str, HConnection)
        gtk.ListStore.__init__(self, *cols) # Call the parent constructeur
        self.watcher = connection_watcher.ConnectionWatcher()
        self.watcher.connect('connection-added', self.connection_added_cb)
        self.connections = []
        self.handle_iter = {}

    def connection_added_cb(self, sender, conn):
        logging.debug("Adding new connection: %s", conn)
        if CONNECTION_INTERFACE_REQUESTS not in conn:
            logging.warning("CONNECTION_INTERFACE_REQUESTS not implemented in %s" %\
                    (conn.service_name))
            return None

        if not conn_can_request_contact_list(conn):
            logging.warning("CHANNEL_TYPE_CONTACT_LIST not requestable in %s" %\
                    (conn.service_name))
            return None

        co = HConnection(conn)
        self.handle_iter[co] = {}
        self.connections.append(co)
        co.connect('contacts_added', self.contacts_added_cb)
        co.connect('contacts_removed', self.contacts_removed_cb)
        co.get_contacts()

    def contacts_added_cb(self, hconn, contacts):
        print 'contacts_added_cb'
        for handle, id, alias, presence, presence_message in contacts:
            if contact_is_online(handle, hconn.conn):
                iter = self.append([handle, id, alias, presence,
                    presence_message, hconn])
                self.handle_iter[hconn][handle] = iter

    def contacts_removed_cb(self, hconn, contacts):
        print 'contacts_removed_cb'
        for handle in contacts:
            if handle in self.handle_int[hconn]:
                iter = self.handle_iter[hconn][handle]
                self.remove(iter)
                del self.handle_iter[hconn][handle]

gobject.type_register(ContactManager)

if __name__ == '__main__':
    w = gtk.Window()
    w.show()
    c = ContactManager()
    hbox = gtk.VBox()
    label = gtk.Label('Select a contact')
    label.show()
    hbox.pack_start(label)

    combobox = gtk.ComboBox(c)
    cell = gtk.CellRendererText()
    combobox.pack_start(cell, True)
    combobox.add_attribute(cell, 'text', 2)
    combobox.show()
    hbox.pack_start(combobox)
    hbox.show()
    w.add(hbox)
    gtk.main()
