import gtk
import invitedialog, util
from boardwidget import BoardWidget
import ui
import tp

class GameWindow:
    "Graphical interface for the game"
    def __init__(self, theme, game = None, side = None, tube = None):
        self.theme = theme
        self.tube = tube

        self.game_finish = False

        builder = gtk.Builder()
        self.builder = builder
        builder.add_from_file(util.find_file("game_window.ui"))
        builder.connect_signals(self)
        self.window = builder.get_object("window1")

        self.window.connect('destroy', self.on_quit)

        box = builder.get_object('stagebox')

        self.board = BoardWidget(self.theme, game, side)
        box.pack_start(self.board.widget)
        self.board.show()

        self.board.game.connect("turn_ended", self.disable_endturn_button_cb)
        self.board.game.connect('player_wins', self.end_game_cb)
        self.board.game.connect('new_game', self.new_game_cb)

        #disable cause not implemented
        self.builder.get_object("undo").set_sensitive(False)
        self.builder.get_object("undo1").set_sensitive(False)
        icone_file = util.find_file("icon.png")
        icone = gtk.gdk.pixbuf_new_from_file(icone_file)
        self.window.set_icon(icone)
        self.window.show_all()

    def on_invite(self, sender):
        if not hasattr(self, 'dialog'):
            self.dialog = invitedialog.InviteDialog()
        contact = self.dialog.run()
        if contact:
            handle, conn = contact
            tp.offer_tube(conn, handle)

    def on_end_turn(self, sender):
        if not self.game_finish:
            self.board.end_turn()

    def on_new_game(self, sender):
        ui.get_ui().new_game_window()

    def on_restart_game(self, sender):
        self.game_finish = False
        self.builder.get_object("endturn_menuitem").set_sensitive(True)
        self.builder.get_object("endturn").set_sensitive(True)
        self.board.game.new_game()

    def on_quit(self, sender):
        # TODO confirmation to avoid unexpected exit

        # If we are in a networked game, close the tube properly
        if self.tube:
            self.tube[tp.CHANNEL_INTERFACE].Close()
        self.window.destroy()

    def on_about(self, sender):
        "Show an about dialog"
        #icons = gtk.icon_theme_get_default()
        #logo_pixbuf = icons.load_icon("gtk-paste", 32, 0)

        about = gtk.AboutDialog()
        about.set_name("Pyhalebarde")
        about.set_version("0.3")
        about.set_license("GPL 2 & 3")
        about.set_comments(_("A adaptation of Halebarde & Gonfagon\n by Eric"\
                " Forgeot"))
        about.set_authors(["Olivier Le Tanh Duong", "Laurent Peuch"])
        #about.set_documenters(constants.documenters)
        about.set_translator_credits(_("translator_credits"))
        about.set_website("http://launchpad.net/pyhalebarde")
        #about.set_logo(logo_pixbuf)

        about.run()
        about.hide()

    def on_help(self, sender):
        "Show the help dialog"
        screen = gtk.gdk.screen_get_default()
        timestamp = gtk.get_current_event_time()
        help_dialog = gtk.show_uri(screen,
                "file://" + util.find_file("readme.html"),
                timestamp)

    def on_undo(self, sender):
        print "not implemented"

    def disable_endturn_button_cb(self, sender, turn, turn_number):
        "On an endturn, if it's not your turn you shouldn't be able to press"\
        "the end turn button"
        print 'turn_ended', turn, turn_number
        print self.board.side

        if self.board.side:
            endturn_button = self.builder.get_object("endturn")
            endturn_menuitem = self.builder.get_object("endturn_menuitem")
            #if it's not our turn
            if turn != self.board.side:
                endturn_menuitem.set_sensitive(False)
                endturn_button.set_sensitive(False)
            else:
                endturn_menuitem.set_sensitive(True)
                endturn_button.set_sensitive(True)

    def end_game_cb(self, sender, player):
        "At the end of the game, disable the end turn button and switch"\
        " self.game_finish"
        self.builder.get_object("endturn").set_sensitive(False)
        self.builder.get_object("endturn_menuitem").set_sensitive(False)

    def new_game_cb(self, sender):
        self.builder.get_object("endturn").set_sensitive(True)
        self.builder.get_object("endturn_menuitem").set_sensitive(True)

