"units of pyhalebarde"
import globals

class Unit(object):
    "game unit template"
    SPEED = 0

    def __init__ (self, side = None):
        self.side = side

    def __str__(self):
        return self.side + ' ' + self.NAME

    def can_reach(self, from_x, from_y, board):
        """
        Algorithm to determin which cases an unit can go on,
        given a certain speed and depending on the terrain.
        
        Plain case count as 1 and Hill cases count as 2
        """
        
        return_list = []
        todo = [(from_x, from_y)]
        dist =  {(from_x, from_y) : 0}

        while todo != [] :
            x, y = todo.pop()
            for i in range(x - 1, x + 2):
                for j in range(y - 1, y + 2):
                    if (i, j) != (x, y) and globals.is_in_battlefield(i,j):
                        if (i, j) in globals.HILLS :
                            d = dist[(x, y)] + 2
                        else :
                            d = dist[(x, y)] + 1
                        if d <= self.SPEED :
                            if (i, j) not in dist.keys() or d < dist[(i, j)]:
                                dist[(i, j)] = d
                                todo.append((i, j))
                            return_list.append((i, j))
        return return_list

    def _can_walk_on(self, x, y):
        "check if a unit can walk on a case, depending if it's a hill or not"
        return self.CAN_GO_ON_HILLS or (x, y) not in globals.HILLS 

    def can_go_from (self, from_x, from_y, board):
        "return a list of the positions where the unit can go"

        if self.side == "Black" and globals.is_in_black_base(from_x, from_y):
            positions = globals.BLACK_BASE_ENTRY
        elif self.side == "White" and globals.is_in_white_base(from_x, from_y):
            positions = globals.WHITE_BASE_ENTRY
        elif globals.is_in_battlefield(from_x, from_y):
            positions = self.can_reach(from_x, from_y, board)
        
        return [(x, y) for x, y in positions if board.empty(x, y) and\
                self._can_walk_on(x, y)]

class Soldier(Unit):
    "a close combat soldier"
    NAME = "Soldier"
    CAN_GO_ON_HILLS = True
    SPEED = 3

    def can_attack_from (self, from_x, from_y, board):
        "return a list of positions the unit can attack"
        def valid_attack(x, y):
            victim = board.board[x][y]
            if not victim or victim.side == self.side:
                return False

            return victim.__class__ !=  Saboteur \
                or victim.can_be_attacked(x, y, board, from_x, from_y)

        return [(x, y) for x, y in self.can_reach(from_x, from_y, board)
                    if valid_attack(x,y)]

class Archer(Unit):
    "a ranged attack fighter"
    NAME = "Archer"
    CAN_GO_ON_HILLS = True
    MOVEMENTS = [(0, 1), (0, 2), (0, -1), (0, -2), (-1, 0), (-2, 0), (1, 0),
            (2, 0), (1, 1), (-1, 1), (-1, -1), (1, -1)]

    def can_reach(self, from_x, from_y, board):
        """Archer unit have different movements rules than other units
        so we redefine the can_reach function here"""
        list = []
        for i, j in self.MOVEMENTS :
            x, y = from_x + i, from_y +j
            if globals.is_in_battlefield(x,y):
                list.append((x, y))
        return list

    def can_attack_from (self, from_x, from_y, board):
        """
        return a list of positions the unit can attack.
        Archer units can attack to there movement range if they are on a hill
        """
        
        if (from_x, from_y) not in globals.HILLS:
            return []

        return [(x, y) for x, y in self.can_reach(from_x, from_y, board) if
                not board.empty(x, y) and board.board[x][y].side != self.side]

class Saboteur(Unit):
    "a non attacking unit, which can sabotage enemy base."
    NAME = "Saboteur"
    CAN_GO_ON_HILLS = False
    SPEED = 2

    def can_attack_from (self, from_x, from_y, board):
        "Saboteur unit can never attack"
        return []

    def can_be_attacked(self, x, y, board, attacker_x, attacker_y):
        """return if the saboteur can be attack
        Concretly we look if there is another enemy unit to support the
        attack of our soldier unit.
        """
        for i in range(x-1, x+2):
            for j in range(y-1, y+2):
                # if it's on the board, is not the attacker and the unit
                # is an enemy of the saboteur
                if globals.is_in_battlefield(i,j)\
                    and not board.empty(i, j)\
                    and board.board[i][j].side != self.side\
                    and (i,j) != (attacker_x, attacker_y):
                    return True
        return False
