#!/usr/bin/env python
# Installation script for pyhalebarde

 #   PyHalebarde, a turn per turn strategy game
 #   Copyright (C) 2008 Laurent P & Olivier Le Thanh Duong

 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 2 of the License, or
 #   (at your option) any later version.

 #   This program is distributed in the hope that it will be useful,
 #   but WITHOUT ANY WARRANTY; without even the implied warranty of
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #   GNU General Public License for more details.

 #   You should have received a copy of the GNU General Public License
 #   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from distutils.core import setup

from distutils.command.build_py import build_py as _build_py
from distutils.command.install_data import install_data as _install_data

import distutils.file_util
import distutils.dir_util

import glob
import os

# create .mo files from .po locales if needed

class build_py(_build_py):
    def run(self):
        for item in os.listdir("po"):
            if item.endswith(".po"):
                lang = item[:-3]
                dest = "%s.mo" % (lang)
                print "compiling '%s' translation '%s'" % (lang, dest)
                os.system("msgfmt po/%s -o po/%s" % (item, dest))

        self.build_doc()
        self.update_pot()
        _build_py.run(self)
    
    def build_doc(self):
        print "Generating readme.html"
        os.system('rst2html README > data/readme.html')

    def update_pot(self):
        output = 'pyhalebarde.pot'
        source_files = ' pyhalebarde halebarde/*.py'
        os.system('xgettext --language=Python --keyword=_ --output=%s %s' 
                    % (output, source_files))

class install_data(_install_data):
# Add the translation (mo files) in the right directory
    def run(self):
        self.data_files.append(("share/games/pyhalebarde", ['data/readme.html']))
        _install_data.run(self)
        self.generate_in_files(self.install_dir)
        self.install_in_files(self.install_dir)
        for item in os.listdir("po"):
            if item.endswith(".mo"):
                lang = item[:-3]
                source = "po/%s" % (item)
                dest_dir = "%s/share/locale/%s/LC_MESSAGES" % (self.install_dir,lang)
                dest = dest_dir + "/pyhalebarde.mo"
                print "Installing translation '%s' to '%s'" % (source, dest)
                distutils.dir_util.mkpath(dest_dir)
                distutils.file_util.copy_file(source, dest)

    def generate_in_files(self, prefix):
        print prefix
        for item in os.listdir("data"):
            if item.endswith(".in"):
                dest = "data/%s" % (item[:-3])
                source = "data/%s" % (item)
                s = open(source, "r")
                d = open(dest, "w")
                d.write(s.read().replace("@bindir@", "%s/games/" % (prefix)))
                d.close()
                s.close()

    def install_in_files(self, prefix):
        # We need to do this manually cause we generate the file here
        # if we simply add it to data_files it won't be found
        source = 'data/org.gnome.Empathy.DTubeHandler.org.pygame.Pyhalebarde.service'
        dest_dir = "%s/share/dbus-1/services" % prefix
        distutils.dir_util.mkpath(dest_dir)
        distutils.file_util.copy_file(source, dest_dir)
        os.system('dbus-send --print-reply --session --type=method_call --dest=org.freedesktop.DBus / org.freedesktop.DBus.ReloadConfig')

INCLUDED_THEME = ['wesnoth', 'retro']
DATA_FILES = []
for theme in INCLUDED_THEME:
    theme_dir = "share/games/pyhalebarde/themes/%s" % (theme)
    theme_files = glob.glob('themes/%s/*' % (theme))
    DATA_FILES.append((theme_dir, theme_files))

DATA_FILES.append(("share/games/pyhalebarde", [
    "pyhalebarde.png", "data/icon.png", "data/game_window.ui",
    "data/invite_dialog.ui"
    ]))
DATA_FILES.append(("share/pixmaps", ["pyhalebarde.png"]))
DATA_FILES.append(("share/applications", ["pyhalebarde.desktop"]))
#DATA_FILES.append(("share/dbus-1/services", ["data/org.gnome.Empathy.DTubeHandler.org.pygame.Pyhalebarde"])) 

#install the script as a data file so it can be placed in /usr/game
DATA_FILES.append(("games", ["pyhalebarde"]))

# install the mo file in the right directory
# TODO

setup(name='pyhalebarde',
      version='0.3',
      description='pyHalebarde game',
      author='Olivier Le Thanh Duong',
      author_email='olethanh@gmail.com',
      url='https://launchpad.net/pyhalebarde',
      cmdclass={'build_py': build_py, 'install_data' : install_data},
      packages=['halebarde'],
      #scripts=['pyhalebarde'],
      data_files = DATA_FILES
     )

